@@ -1,11 +1,7 @@
-"""
-    WeasyPrint
-    ==========
+"""The Awesome Document Factory.
 
-    WeasyPrint converts web documents to PDF.
-
-    The public API is what is accessible from this "root" packages
-    without importing sub-modules.
+The public API is what is accessible from this "root" packages without
+importing sub-modules.
 
 """
 
@@ -16,7 +12,7 @@ import cssselect2
 import html5lib
 import tinycss2
 
-VERSION = __version__ = '54.3'
+VERSION = __version__ = '55.0'
 
 __all__ = [
     'HTML', 'CSS', 'Attachment', 'Document', 'Page', 'default_url_fetcher',
@@ -113,8 +109,6 @@ class HTML:
         access to individual pages and various meta-data.
         See :meth:`write_pdf` to get a PDF directly.
 
-        .. versionadded:: 0.15
-
         :param list stylesheets:
             An optional list of user stylesheets. List elements are
             :class:`CSS` objects, filenames, URLs, or file
@@ -236,8 +230,6 @@ class CSS:
 class Attachment:
     """File attachment for a PDF document.
 
-    .. versionadded:: 0.22
-
     An instance is created in the same way as :class:`HTML`, except that the
     HTML specific arguments (``encoding`` and ``media_type``) are not
     supported. An optional description can be provided with the ``description``
@@ -251,8 +243,8 @@ class Attachment:
                  string=None, base_url=None, url_fetcher=default_url_fetcher,
                  description=None):
         self.source = _select_source(
-            guess, filename, url, file_obj, string,
-            base_url=base_url, url_fetcher=url_fetcher)
+            guess, filename, url, file_obj, string, base_url=base_url,
+            url_fetcher=url_fetcher)
         self.description = description
 
 
@@ -268,8 +260,8 @@ def _select_source(guess=None, filename=None, url=None, file_obj=None,
         param for param in (guess, filename, url, file_obj, string) if
         param is not None]
     if len(selected_params) != 1:
-        raise TypeError('Expected exactly one source, got ' + (
-            ', '.join(selected_params) or 'nothing'))
+        source = ', '.join(selected_params) or 'nothing'
+        raise TypeError(f'Expected exactly one source, got {source}')
     elif guess is not None:
         if hasattr(guess, 'read'):
             type_ = 'file_obj'
@@ -286,8 +278,6 @@ def _select_source(guess=None, filename=None, url=None, file_obj=None,
         with result as result:
             yield result
     elif filename is not None:
-        if isinstance(filename, Path):
-            filename = str(filename)
         if base_url is None:
             base_url = path2url(filename)
         with open(filename, 'rb') as file_obj:
