@@ -1,16 +1,10 @@
-"""
-    weasyprint.draw
-    ---------------
-
-    Take an "after layout" box tree and draw it onto a pydyf stream.
-
-"""
+"""Take an "after layout" box tree and draw it onto a pydyf stream."""
 
 import contextlib
 import operator
 from colorsys import hsv_to_rgb, rgb_to_hsv
 from io import BytesIO
-from math import ceil, floor, pi, sqrt, tan
+from math import ceil, cos, floor, pi, sin, sqrt, tan
 from xml.etree import ElementTree
 
 from PIL import Image
@@ -19,6 +13,7 @@ from .formatting_structure import boxes
 from .images import RasterImage, SVGImage
 from .layout import replaced
 from .layout.background import BackgroundLayer
+from .matrix import Matrix
 from .stacking import StackingContext
 from .text.ffi import ffi, harfbuzz, pango, units_from_double, units_to_double
 from .text.line_break import get_last_word_end
@@ -174,6 +169,7 @@ def draw_stacking_context(stream, stacking_context):
             # Point 7
             for block in [box] + stacking_context.blocks_and_cells:
                 if isinstance(block, boxes.ReplacedBox):
+                    draw_border(stream, block)
                     draw_replacedbox(stream, block)
                 else:
                     for child in block.children:
@@ -363,8 +359,6 @@ def draw_table_backgrounds(stream, page, table):
 
 
 def draw_background_image(stream, layer, image_rendering):
-    from .document import Matrix
-
     if layer.image is None or 0 in layer.size:
         return
 
@@ -378,8 +372,10 @@ def draw_background_image(stream, layer, image_rendering):
 
     if repeat_x == 'no-repeat':
         # We want at least the whole image_width drawn on sub_surface, but we
-        # want to be sure it will not be repeated on the painting_width.
-        repeat_width = max(image_width, painting_width)
+        # want to be sure it will not be repeated on the painting_width. We
+        # double the painting width to ensure viewers don't incorrectly bleed
+        # the edge of the pattern into the painting area. (See #1539.)
+        repeat_width = max(image_width, 2 * painting_width)
     elif repeat_x in ('repeat', 'round'):
         # We repeat the image each image_width.
         repeat_width = image_width
@@ -399,7 +395,7 @@ def draw_background_image(stream, layer, image_rendering):
 
     # Comments above apply here too.
     if repeat_y == 'no-repeat':
-        repeat_height = max(image_height, painting_height)
+        repeat_height = max(image_height, 2 * painting_height)
     elif repeat_y in ('repeat', 'round'):
         repeat_height = image_height
     else:
@@ -412,11 +408,6 @@ def draw_background_image(stream, layer, image_rendering):
         else:
             repeat_height = image_height
 
-    if repeat_x == repeat_y == 'no-repeat':
-        # PDF patterns always repeat, use a big number to hide repetition
-        repeat_width = 2 * stream.page_rectangle[2]
-        repeat_height = 2 * stream.page_rectangle[3]
-
     matrix = Matrix(e=position_x + positioning_x, f=position_y + positioning_y)
     matrix @= stream.ctm
     pattern = stream.add_pattern(
@@ -1009,16 +1000,19 @@ def draw_text(stream, textbox, offset_x, text_overflow, block_ellipsis):
         thickness = textbox.pango_layout.underline_thickness
         offset_y = (
             textbox.baseline - textbox.pango_layout.ascent + thickness / 2)
+        draw_text_decoration(
+            stream, textbox, offset_x, offset_y, thickness, color)
     if 'underline' in values:
         thickness = textbox.pango_layout.underline_thickness
         offset_y = (
             textbox.baseline - textbox.pango_layout.underline_position +
             thickness / 2)
+        draw_text_decoration(
+            stream, textbox, offset_x, offset_y, thickness, color)
     if 'line-through' in values:
         thickness = textbox.pango_layout.strikethrough_thickness
         offset_y = (
             textbox.baseline - textbox.pango_layout.strikethrough_position)
-    if values != 'none':
         draw_text_decoration(
             stream, textbox, offset_x, offset_y, thickness, color)
 
@@ -1026,16 +1020,20 @@ def draw_text(stream, textbox, offset_x, text_overflow, block_ellipsis):
 
 
 def draw_emojis(stream, font_size, x, y, emojis):
-    for emoji in emojis:
-        for image, font, a, d, e, f in emojis:
-            stream.push_state()
-            stream.transform(a=a, d=d, e=x + e * font_size, f=y + f)
-            image.draw(stream, font_size, font_size, None)
-            stream.pop_state()
+    for image, font, a, d, e, f in emojis:
+        stream.push_state()
+        stream.transform(a=a, d=d, e=x + e * font_size, f=y + f)
+        image.draw(stream, font_size, font_size, None)
+        stream.pop_state()
 
 
-def draw_first_line(stream, textbox, text_overflow, block_ellipsis, x, y):
+def draw_first_line(stream, textbox, text_overflow, block_ellipsis, x, y,
+                    angle=0):
     """Draw the given ``textbox`` line to the document ``stream``."""
+    font_size = textbox.style['font_size']
+    if font_size < 1e-6:  # Default float precision used by pydyf
+        return []
+
     pango.pango_layout_set_single_paragraph_mode(
         textbox.pango_layout.layout, True)
 
@@ -1065,10 +1063,10 @@ def draw_first_line(stream, textbox, text_overflow, block_ellipsis, x, y):
 
             textbox.pango_layout.set_text(new_text + ellipsis)
 
-    first_line, second_line = textbox.pango_layout.get_first_line()
+    first_line, index = textbox.pango_layout.get_first_line()
 
     if block_ellipsis != 'none':
-        while second_line:
+        while index:
             last_word_end = get_last_word_end(
                 textbox.pango_layout.text[:-len(ellipsis)],
                 textbox.style['lang'])
@@ -1076,20 +1074,22 @@ def draw_first_line(stream, textbox, text_overflow, block_ellipsis, x, y):
                 break
             new_text = textbox.pango_layout.text[:last_word_end]
             textbox.pango_layout.set_text(new_text + ellipsis)
-            first_line, second_line = textbox.pango_layout.get_first_line()
+            first_line, index = textbox.pango_layout.get_first_line()
 
-    font_size = textbox.style['font_size']
-    utf8_text = textbox.pango_layout.text.encode('utf-8')
+    utf8_text = textbox.pango_layout.text.encode()
     previous_utf8_position = 0
 
     runs = [first_line.runs[0]]
     while runs[-1].next != ffi.NULL:
         runs.append(runs[-1].next)
 
-    stream.text_matrix(font_size, 0, 0, -font_size, x, y)
+    matrix = Matrix(font_size, 0, 0, -font_size, x, y)
+    if angle:
+        matrix = Matrix(a=cos(angle), b=-sin(angle),
+                        c=sin(angle), d=cos(angle)) @ matrix
+    stream.text_matrix(*matrix.values)
     last_font = None
     string = ''
-    fonts = stream.get_fonts()
     x_advance = 0
     emojis = []
     for run in runs:
@@ -1103,19 +1103,7 @@ def draw_first_line(stream, textbox, text_overflow, block_ellipsis, x, y):
 
         # Font content
         pango_font = glyph_item.item.analysis.font
-        hb_font = pango.pango_font_get_hb_font(pango_font)
-        hb_face = harfbuzz.hb_font_get_face(hb_font)
-        font_hash = hash(hb_face)
-        if font_hash in fonts:
-            font = fonts[font_hash]
-        else:
-            hb_blob = ffi.gc(
-                harfbuzz.hb_face_reference_blob(hb_face),
-                harfbuzz.hb_blob_destroy)
-            hb_data = harfbuzz.hb_blob_get_data(hb_blob, stream.length)
-            file_content = ffi.unpack(hb_data, int(stream.length[0]))
-            index = harfbuzz.hb_face_get_index(hb_face)
-            font = stream.add_font(font_hash, file_content, pango_font, index)
+        font = stream.add_font(pango_font)
 
         # Positions of the glyphs in the UTF-8 string
         utf8_positions = [offset + clusters[i] for i in range(1, num_glyphs)]
@@ -1176,10 +1164,12 @@ def draw_first_line(stream, textbox, text_overflow, block_ellipsis, x, y):
             # Mapping between glyphs and characters
             if glyph not in font.cmap:
                 utf8_slice = slice(previous_utf8_position, utf8_position)
-                font.cmap[glyph] = utf8_text[utf8_slice].decode('utf-8')
+                font.cmap[glyph] = utf8_text[utf8_slice].decode()
             previous_utf8_position = utf8_position
 
             if font.svg:
+                hb_font = pango.pango_font_get_hb_font(pango_font)
+                hb_face = harfbuzz.hb_font_get_face(hb_font)
                 hb_blob = ffi.gc(
                     harfbuzz.hb_ot_color_glyph_reference_svg(hb_face, glyph),
                     harfbuzz.hb_blob_destroy)
@@ -1191,6 +1181,7 @@ def draw_first_line(stream, textbox, text_overflow, block_ellipsis, x, y):
                     a = d = font.widths[glyph] / 1000 / font.upem * font_size
                     emojis.append([image, font, a, d, x_advance, 0])
             elif font.png:
+                hb_font = pango.pango_font_get_hb_font(pango_font)
                 hb_blob = ffi.gc(
                     harfbuzz.hb_ot_color_glyph_reference_png(hb_font, glyph),
                     harfbuzz.hb_blob_destroy)
