@@ -1,15 +1,10 @@
-"""
-    weasyprint.layout.inline
-    ------------------------
-
-    Line breaking and layout for inline-level boxes.
-
-"""
+"""Layout for inline-level boxes."""
 
 import unicodedata
+from math import inf
 
 from ..css import computed_from_cascaded
-from ..css.computed_values import ex_ratio, strut_layout
+from ..css.computed_values import character_ratio, strut_layout
 from ..formatting_structure import boxes
 from ..text.line_break import can_break_text, create_layout, split_first_line
 from .absolute import AbsolutePlaceholder, absolute_layout
@@ -192,7 +187,6 @@ def skip_first_whitespace(box, skip_stack):
         index = 0
         next_skip_stack = None
     else:
-        # TODO: handle multiple skip stacks
         (index, next_skip_stack), = skip_stack.items()
 
     if isinstance(box, boxes.TextBox):
@@ -303,7 +297,6 @@ def first_letter_to_box(box, skip_stack, first_letter_style):
             elif child.text:
                 character_found = False
                 if skip_stack:
-                    # TODO: handle multiple skip stacks
                     child_skip_stack, = skip_stack.values()
                     if child_skip_stack:
                         index, = child_skip_stack
@@ -353,7 +346,6 @@ def first_letter_to_box(box, skip_stack, first_letter_style):
                             index: {child_index + 1: grandchild_skip_stack}}
         elif isinstance(child, boxes.ParentBox):
             if skip_stack:
-                # TODO: handle multiple skip stacks
                 child_skip_stack, = skip_stack.values()
             else:
                 child_skip_stack = None
@@ -406,7 +398,7 @@ def inline_block_box_layout(context, box, position_x, skip_stack,
     box.position_x = position_x
     box.position_y = 0
     box, _, _, _, _ = block_container_layout(
-        context, box, bottom_space=-float('inf'), skip_stack=skip_stack,
+        context, box, bottom_space=-inf, skip_stack=skip_stack,
         page_is_empty=True, absolute_boxes=absolute_boxes,
         fixed_boxes=fixed_boxes, adjoining_margins=None, discard=False)
     box.baseline = inline_block_baseline(box)
@@ -472,8 +464,10 @@ def split_inline_level(context, box, position_x, max_x, bottom_space,
             skip = skip or 0
             assert skip_stack is None
 
+        is_line_start = len(line_children) == 0
         new_box, skip, preserved_line_break = split_text_box(
-            context, box, max_x - position_x, skip)
+            context, box, max_x - position_x, skip,
+            is_line_start=is_line_start)
 
         if skip is None:
             resume_at = None
@@ -515,8 +509,8 @@ def split_inline_level(context, box, position_x, max_x, bottom_space,
             if getattr(box, f'margin_{side}') == 'auto':
                 setattr(box, f'margin_{side}', 0)
         new_box, resume_at, _, _, _ = flex_layout(
-            context, box, -float('inf'), skip_stack, containing_block,
-            False, absolute_boxes, fixed_boxes)
+            context, box, -inf, skip_stack, containing_block, False,
+            absolute_boxes, fixed_boxes)
         preserved_line_break = False
         first_letter = '\u2e80'
         last_letter = '\u2e80'
@@ -714,7 +708,6 @@ def split_inline_box(context, box, position_x, max_x, bottom_space, skip_stack,
     if is_start:
         skip = 0
     else:
-        # TODO: handle multiple skip stacks
         (skip, skip_stack), = skip_stack.items()
 
     for i, child in enumerate(box.children[skip:]):
@@ -891,7 +884,7 @@ def split_inline_box(context, box, position_x, max_x, bottom_space, skip_stack,
         float_widths)
 
 
-def split_text_box(context, box, available_width, skip):
+def split_text_box(context, box, available_width, skip, is_line_start=True):
     """Keep as much text as possible from a TextBox in a limited width.
 
     Try not to overflow but always have some text in ``new_box``.
@@ -909,7 +902,8 @@ def split_text_box(context, box, available_width, skip):
     if font_size == 0 or not text:
         return None, None, False
     layout, length, resume_index, width, height, baseline = split_first_line(
-        text, box.style, context, available_width, box.justification_spacing)
+        text, box.style, context, available_width, box.justification_spacing,
+        is_line_start=is_line_start)
     assert resume_index != 0
 
     # Convert ``length`` and ``resume_at`` from UTF-8 indexes in text
@@ -918,11 +912,11 @@ def split_text_box(context, box, available_width, skip):
     # resume_at is not set). One code point is one or more byte, so
     # UTF-8 indexes are always bigger or equal to Unicode indexes.
     new_text = layout.text
-    encoded = text.encode('utf8')
+    encoded = text.encode()
     if resume_index is not None:
-        between = encoded[length:resume_index].decode('utf8')
-        resume_index = len(encoded[:resume_index].decode('utf8'))
-    length = len(encoded[:length].decode('utf8'))
+        between = encoded[length:resume_index].decode()
+        resume_index = len(encoded[:resume_index].decode())
+    length = len(encoded[:length].decode())
 
     if length > 0:
         box = box.copy_with_text(new_text)
@@ -1054,8 +1048,8 @@ def inline_box_verticality(box, top_bottom_subtrees, baseline_y):
         if vertical_align == 'baseline':
             child_baseline_y = baseline_y
         elif vertical_align == 'middle':
-            one_ex = box.style['font_size'] * ex_ratio(box.style)
-            top = baseline_y - (one_ex + child.margin_height()) / 2.
+            one_ex = box.style['font_size'] * character_ratio(box.style, 'x')
+            top = baseline_y - (one_ex + child.margin_height()) / 2
             child_baseline_y = top + child.baseline
         elif vertical_align == 'text-top':
             # align top with the top of the parent’s content area
@@ -1170,8 +1164,7 @@ def add_word_spacing(context, box, justification_spacing, x_advance):
         nb_spaces = count_spaces(box)
         if nb_spaces > 0:
             layout = create_layout(
-                box.text, box.style, context, -float('inf'),
-                box.justification_spacing)
+                box.text, box.style, context, -inf, box.justification_spacing)
             layout.deactivate()
             extra_space = justification_spacing * nb_spaces
             x_advance += extra_space
