@@ -1,12 +1,7 @@
-"""
-    weasyprint.layout.pages
-    -----------------------
-
-    Layout for pages and CSS3 margin boxes.
-
-"""
+"""Layout for pages and CSS3 margin boxes."""
 
 import copy
+from math import inf
 
 from ..css import PageType, computed_from_cascaded
 from ..formatting_structure import boxes, build
@@ -435,9 +430,9 @@ def margin_box_content_layout(context, page, box):
     """Layout a margin box’s content once the box has dimensions."""
     positioned_boxes = []
     box, resume_at, next_page, _, _ = block_container_layout(
-        context, box, bottom_space=-float('inf'), skip_stack=None,
-        page_is_empty=True, absolute_boxes=positioned_boxes,
-        fixed_boxes=positioned_boxes, adjoining_margins=None, discard=False)
+        context, box, bottom_space=-inf, skip_stack=None, page_is_empty=True,
+        absolute_boxes=positioned_boxes, fixed_boxes=positioned_boxes,
+        adjoining_margins=None, discard=False)
     assert resume_at is None
     for absolute_box in positioned_boxes:
         absolute_layout(
@@ -561,8 +556,8 @@ def make_page(context, root_box, page_type, resume_at, page_number,
         reported_footnote_area = build.create_anonymous_boxes(
             footnote_area.deepcopy())
         reported_footnote_area, _, _, _, _ = block_level_layout(
-            context, reported_footnote_area, -float('inf'), None,
-            footnote_area.page, True, [], [], [], False)
+            context, reported_footnote_area, -inf, None, footnote_area.page,
+            True, [], [], [], False)
         footnote_area.height = reported_footnote_area.height
         context.page_bottom -= reported_footnote_area.margin_height()
 
@@ -596,8 +591,8 @@ def make_page(context, root_box, page_type, resume_at, page_number,
 
     footnote_area = build.create_anonymous_boxes(footnote_area.deepcopy())
     footnote_area, _, _, _, _ = block_level_layout(
-        context, footnote_area, -float('inf'), None, footnote_area.page,
-        True, positioned_boxes, positioned_boxes, [], False)
+        context, footnote_area, -inf, None, footnote_area.page, True,
+        positioned_boxes, positioned_boxes, [], False)
     footnote_area.translate(dy=-footnote_area.margin_height())
 
     page.fixed_boxes = [
@@ -648,8 +643,7 @@ def make_page(context, root_box, page_type, resume_at, page_number,
         # string-set and bookmark-labels don't create boxes, only `content`
         # requires another call to make_page. There is maximum one 'content'
         # item per box.
-        # TODO: remove attribute or set a default value in Box class
-        if hasattr(child, 'missing_link'):
+        if child.missing_link:
             # A CounterLookupItem exists for the css-token 'content'
             counter_lookup = target_collector.counter_lookup_items.get(
                 (child.missing_link, 'content'))
@@ -862,6 +856,7 @@ def make_all_pages(context, root_box, html, pages):
 
         i += 1
         if resume_at is None and not context.reported_footnotes:
-            # Throw away obsolete pages
+            # Throw away obsolete pages and broken out-of-flow boxes
             context.page_maker = context.page_maker[:i + 1]
+            context.broken_out_of_flow.clear()
             return
